package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.BatchDetails;
import digital.laboratory.platform.reagent.service.BatchDetailsService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * 批次明细
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 批次明细 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/batch_details" )
@Api(value = "batch_details", tags = "批次明细管理")
public class BatchDetailsController {

    private final  BatchDetailsService batchDetailsService;

	/**
	 * 通过id查询批次明细
	 * @param batchDetailsId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{batchDetailsId}" )
	@PreAuthorize("@pms.hasPermission('reagent_batch_details_get')" )
	public R<BatchDetails> getById(@PathVariable("batchDetailsId" ) String batchDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        BatchDetails batchDetails = batchDetailsService.getById(batchDetailsId);
		return R.ok(batchDetails);
		//return R.ok(batchDetailsService.getById(batchDetailsId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param batchDetails 批次明细
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_batch_details_get')" )
    public R<IPage<BatchDetails>> getBatchDetailsPage(Page<BatchDetails> page, BatchDetails batchDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<BatchDetails> batchDetailsSList = batchDetailsService.page(page, Wrappers.<BatchDetails>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(batchDetailsSList);
//		return R.ok(batchDetailsService.page(page, Wrappers.query(batchDetails)));
    }


    /**
     * 新增批次明细
     * @param batchDetails 批次明细
     * @return R
     */
    @ApiOperation(value = "新增批次明细", notes = "新增批次明细")
    @SysLog("新增批次明细" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_batch_details_add')" )
    public R<BatchDetails> postAddObject(@RequestBody BatchDetails batchDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());
		if (batchDetailsService.save(batchDetails)) {
			return R.ok(batchDetails, "对象创建成功");
		}
		else {
			return R.failed(batchDetails, "对象创建失败");
		}
    }

    /**
     * 修改批次明细
     * @param batchDetails 批次明细
     * @return R
     */
    @ApiOperation(value = "修改批次明细", notes = "修改批次明细")
    @SysLog("修改批次明细" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_batch_details_edit')" )
    public R<BatchDetails> putUpdateById(@RequestBody BatchDetails batchDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (batchDetailsService.updateById(batchDetails)) {
			return R.ok(batchDetails, "保存对象成功");
		}
		else {
			return R.failed(batchDetails, "保存对象失败");
		}
    }

    /**
     * 通过id删除批次明细
     * @param batchDetailsId id
     * @return R
     */
    @ApiOperation(value = "通过id删除批次明细", notes = "通过id删除批次明细")
    @SysLog("通过id删除批次明细" )
    @DeleteMapping("/{batchDetailsId}" )
    @PreAuthorize("@pms.hasPermission('reagent_batch_details_del')" )
    public R<BatchDetails> deleteById(@PathVariable String batchDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        BatchDetails oldBatchDetails = batchDetailsService.getById(batchDetailsId);

		if (batchDetailsService.removeById(batchDetailsId)) {
			return R.ok(oldBatchDetails, "对象删除成功");
		}
		else {
			return R.failed(oldBatchDetails, "对象删除失败");
		}

    }
	@PutMapping("/update/data")
	public R updateCabinetData(){
		batchDetailsService.updateCabinetData();
		return R.ok("更新数据成功");
	}

}
